@@ -358,6 +358,7 @@ inner_ref: {
 
 issue_2107: {
     options = {
+        assignments: true,
         collapse_vars: true,
         inline: true,
         passes: 3,
@@ -387,6 +388,7 @@ issue_2107: {
 
 issue_2114_1: {
     options = {
+        assignments: true,
         collapse_vars: true,
         if_return: true,
         inline: true,
@@ -419,6 +421,7 @@ issue_2114_1: {
 
 issue_2114_2: {
     options = {
+        assignments: true,
         collapse_vars: true,
         if_return: true,
         inline: true,
@@ -1223,6 +1226,7 @@ issue_2630_1: {
 
 issue_2630_2: {
     options = {
+        assignments: true,
         collapse_vars: true,
         inline: true,
         passes: 2,
@@ -1320,6 +1324,7 @@ issue_2630_4: {
 
 issue_2630_5: {
     options = {
+        assignments: true,
         collapse_vars: true,
         inline: true,
         reduce_vars: true,
@@ -1398,6 +1403,8 @@ recursive_inline_2: {
 issue_2657: {
     options = {
         inline: true,
+        passes: 2,
+        reduce_funcs: true,
         reduce_vars: true,
         sequences: true,
         unused: true,
@@ -2315,3 +2322,356 @@ issue_3125: {
     }
     expect_stdout: "PASS"
 }
+
+issue_3274: {
+    options = {
+        collapse_vars: true,
+        inline: true,
+        join_vars: true,
+        loops: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function() {
+            var g = function(a) {
+                var c = a.p, b = c;
+                return b != c;
+            };
+            while (g(1))
+                console.log("FAIL");
+            console.log("PASS");
+        })();
+    }
+    expect: {
+        (function() {
+            for (var c; void 0, (c = 1..p) != c;)
+                console.log("FAIL");
+            console.log("PASS");
+        })();
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3297_1: {
+    options = {
+        collapse_vars: true,
+        inline: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    mangle = {}
+    input: {
+        function function1() {
+            var r = {
+                function2: function2
+            };
+            function function2() {
+                alert(1234);
+                function function3() {
+                    function2();
+                };
+                function3();
+            }
+            return r;
+        }
+    }
+    expect: {
+        function function1() {
+            return {
+                function2: function n() {
+                    alert(1234);
+                    function t() {
+                        n();
+                    }
+                    t();
+                }
+            };
+        }
+    }
+}
+
+issue_3297_2: {
+    options = {
+        collapse_vars: true,
+        conditionals: true,
+        inline: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    mangle = {}
+    input: {
+        function function1(session) {
+            var public = {
+                processBulk: processBulk
+            };
+            return public;
+            function processBulk(bulk) {
+                var subparam1 = session();
+                function processOne(param1) {
+                    var param2 = {
+                        subparam1: subparam1
+                    };
+                    doProcessOne({
+                        param1: param1,
+                        param2: param2,
+                    }, function () {
+                        processBulk(bulk);
+                    });
+                };
+                if (bulk && bulk.length > 0)
+                    processOne(bulk.shift());
+            }
+            function doProcessOne(config, callback) {
+                console.log(JSON.stringify(config));
+                callback();
+            }
+        }
+        function1(function session() {
+            return 42;
+        }).processBulk([1, 2, 3]);
+    }
+    expect: {
+        function function1(o) {
+            return {
+                processBulk: function t(u) {
+                    var r = o();
+                    function n(n) {
+                        var o = {
+                            subparam1: r
+                        };
+                        c({
+                            param1: n,
+                            param2: o
+                        }, function() {
+                            t(u);
+                        });
+                    }
+                    u && u.length > 0 && n(u.shift());
+                }
+            };
+            function c(n, o) {
+                console.log(JSON.stringify(n));
+                o();
+            }
+        }
+        function1(function() {
+            return 42;
+        }).processBulk([ 1, 2, 3 ]);
+    }
+    expect_stdout: [
+        '{"param1":1,"param2":{"subparam1":42}}',
+        '{"param1":2,"param2":{"subparam1":42}}',
+        '{"param1":3,"param2":{"subparam1":42}}',
+    ]
+}
+
+issue_3297_3: {
+    options = {
+        collapse_vars: true,
+        comparisons: true,
+        conditionals: true,
+        inline: true,
+        join_vars: true,
+        passes: 3,
+        reduce_funcs: true,
+        reduce_vars: true,
+        sequences: true,
+        side_effects: true,
+        unused: true,
+    }
+    mangle = {}
+    input: {
+        function function1(session) {
+            var public = {
+                processBulk: processBulk
+            };
+            return public;
+            function processBulk(bulk) {
+                var subparam1 = session();
+                function processOne(param1) {
+                    var param2 = {
+                        subparam1: subparam1
+                    };
+                    doProcessOne({
+                        param1: param1,
+                        param2: param2,
+                    }, function () {
+                        processBulk(bulk);
+                    });
+                };
+                if (bulk && bulk.length > 0)
+                    processOne(bulk.shift());
+            }
+            function doProcessOne(config, callback) {
+                console.log(JSON.stringify(config));
+                callback();
+            }
+        }
+        function1(function session() {
+            return 42;
+        }).processBulk([1, 2, 3]);
+    }
+    expect: {
+        function function1(u) {
+            return {
+                processBulk: function n(r) {
+                    var o, t = u();
+                    r && 0 < r.length && (o = {
+                        param1: r.shift(),
+                        param2: {
+                            subparam1: t
+                        }
+                    },
+                    console.log(JSON.stringify(o)),
+                    n(r));
+                }
+            };
+        }
+        function1(function() {
+            return 42;
+        }).processBulk([ 1, 2, 3 ]);
+    }
+    expect_stdout: [
+        '{"param1":1,"param2":{"subparam1":42}}',
+        '{"param1":2,"param2":{"subparam1":42}}',
+        '{"param1":3,"param2":{"subparam1":42}}',
+    ]
+}
+
+cross_references_1: {
+    options = {
+        inline: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        var Math = {
+            square: function(n) {
+                return n * n;
+            }
+        };
+        console.log((function(factory) {
+            return factory();
+        })(function() {
+            return function(Math) {
+                return function(n) {
+                    return Math.square(n);
+                };
+            }(Math);
+        })(3));
+    }
+    expect: {
+        var Math = {
+            square: function(n) {
+                return n * n;
+            }
+        };
+        console.log(function(Math) {
+            return function(n) {
+                return Math.square(n);
+            };
+        }(Math)(3));
+    }
+    expect_stdout: "9"
+}
+
+cross_references_2: {
+    options = {
+        collapse_vars: true,
+        evaluate: true,
+        hoist_props: true,
+        inline: true,
+        passes: 4,
+        pure_getters: true,
+        reduce_vars: true,
+        sequences: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var Math = {
+            square: function(n) {
+                return n * n;
+            }
+        };
+        console.log((function(factory) {
+            return factory();
+        })(function() {
+            return function(Math) {
+                return function(n) {
+                    return Math.square(n);
+                };
+            }(Math);
+        })(3));
+    }
+    expect: {
+        console.log(9);
+    }
+    expect_stdout: "9"
+}
+
+cross_references_3: {
+    options = {
+        inline: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        var Math = {
+            square: function(n) {
+                return n * n;
+            },
+            cube: function(n) {
+                return n * n * n;
+            }
+        };
+        console.log(function(factory) {
+            return factory();
+        }(function() {
+            return function(Math) {
+                return function(n) {
+                    Math = {
+                        square: function(x) {
+                            return "(SQUARE" + x + ")";
+                        },
+                        cube: function(x) {
+                            return "(CUBE" + x + ")";
+                        }
+                    };
+                    return Math.square(n) + Math.cube(n);
+                };
+            }(Math);
+        })(2));
+        console.log(Math.square(3), Math.cube(3));
+    }
+    expect: {
+        var Math = {
+            square: function(n) {
+                return n * n;
+            },
+            cube: function(n) {
+                return n * n * n;
+            }
+        };
+        console.log(function(Math) {
+            return function(n) {
+                Math = {
+                    square: function(x) {
+                        return "(SQUARE" + x + ")";
+                    },
+                    cube: function(x) {
+                        return "(CUBE" + x + ")";
+                    }
+                };
+                return Math.square(n) + Math.cube(n);
+            };
+        }(Math)(2));
+        console.log(Math.square(3), Math.cube(3));
+    }
+    expect_stdout: [
+        "(SQUARE2)(CUBE2)",
+        "9 27",
+    ]
+}
